@@ -22,6 +22,7 @@
 
 #include <string.h>
 #include "e-book-backend-couchdb.h"
+#include <libedata-book/e-book-backend-sexp.h>
 #include <libedata-book/e-data-book.h>
 #include <libedata-book/e-data-book-view.h>
 #include <dbus/dbus-glib.h>
@@ -34,25 +35,25 @@
 G_DEFINE_TYPE (EBookBackendCouchDB, e_book_backend_couchdb, E_TYPE_BOOK_BACKEND);
 
 static char *
-vcard_from_couch_document (CouchDBDocument *document)
+vcard_from_couch_document (CouchdbDocument *document)
 {
 	EContact *contact;
 	char *str;
 	GSList *list, *sl;
 	GList *attr_list;
-	CouchDBStructField *app_annotations;
+	CouchdbStructField *app_annotations;
 
 	if (!couchdb_document_is_contact (document))
 		return NULL;
 
 	/* Check if the contact is marked for deletion */
 	if ((app_annotations = couchdb_document_get_application_annotations (document))) {
-		CouchDBStructField *u1_annotations;
+		CouchdbStructField *u1_annotations;
 
 		u1_annotations = couchdb_struct_field_get_struct_field (
 			app_annotations, "Ubuntu One");
 		if (u1_annotations != NULL) {
-			CouchDBStructField *private_annotations;
+			CouchdbStructField *private_annotations;
 
 			private_annotations = couchdb_struct_field_get_struct_field (
 				u1_annotations, "private_application_annotations");
@@ -104,7 +105,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 	while (list != NULL) {
 		const char *email_str, *description_str, *uuid_str;
 		EVCardAttribute *attr;
-		CouchDBStructField *email_address = (CouchDBStructField *) list->data;
+		CouchdbStructField *email_address = (CouchdbStructField *) list->data;
 
 		email_str = couchdb_document_contact_email_get_address (email_address);
 		description_str = couchdb_document_contact_email_get_description (email_address);
@@ -150,7 +151,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 	while (list != NULL) {
 		const char *phone_str, *description_str, *uuid_str;
 		EVCardAttribute *attr;
-		CouchDBStructField *phone_number = (CouchDBStructField *) list->data;
+		CouchdbStructField *phone_number = (CouchdbStructField *) list->data;
 
 		phone_str = couchdb_document_contact_phone_get_number (phone_number);
 		description_str = couchdb_document_contact_phone_get_description (phone_number);
@@ -263,7 +264,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 		char **street_lines;
 		const char *description_str;
 		EContactAddress *contact_address;
-		CouchDBStructField *address = (CouchDBStructField *) list->data;
+		CouchdbStructField *address = (CouchdbStructField *) list->data;
 
 		contact_address = g_new0 (EContactAddress, 1);
 		contact_address->address_format = g_strdup ("");
@@ -303,7 +304,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 	while (list != NULL) {
 		const char *description_str, *address_str, *uuid_str;
 		EVCardAttribute *attr;
-		CouchDBStructField *url = (CouchDBStructField *) list->data;
+		CouchdbStructField *url = (CouchdbStructField *) list->data;
 
 		address_str = couchdb_document_contact_url_get_address (url);
 		description_str = couchdb_document_contact_url_get_description (url);
@@ -337,7 +338,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 	while (list != NULL) {
 		const char *address_str, *description_str, *protocol_str, *uuid_str;
 		EVCardAttribute *attr = NULL;
-		CouchDBStructField *im = (CouchDBStructField *) list->data;
+		CouchdbStructField *im = (CouchdbStructField *) list->data;
 
 		address_str = couchdb_document_contact_im_get_address (im);
 		description_str = couchdb_document_contact_im_get_description (im);
@@ -423,7 +424,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 
 	/* application annotations */
 	if (couchdb_document_has_field (document, "application_annotations")) {
-		CouchDBStructField *annotations = couchdb_document_get_application_annotations (document);
+		CouchdbStructField *annotations = couchdb_document_get_application_annotations (document);
 
 		str = couchdb_struct_field_to_string (annotations);
 		e_vcard_add_attribute_with_value (E_VCARD (contact),
@@ -442,7 +443,7 @@ vcard_from_couch_document (CouchDBDocument *document)
 	return str;
 }
 
-static CouchDBStructField *
+static CouchdbStructField *
 contact_email_to_struct_field (EVCardAttribute *attr)
 {
 	const gchar *email;
@@ -483,12 +484,12 @@ contact_email_to_struct_field (EVCardAttribute *attr)
 	return couchdb_document_contact_email_new (uuid, email, description);
 }
 
-static CouchDBStructField *
+static CouchdbStructField *
 contact_phone_to_struct_field (EVCardAttribute *attr)
 {
 	const gchar *phone;
 	GList *params, *pl;
-	CouchDBStructField *sf;
+	CouchdbStructField *sf;
 	gchar *final_description = NULL;
 	const gchar *description = NULL, *uuid = NULL, *kind = NULL;
 
@@ -553,10 +554,10 @@ contact_phone_to_struct_field (EVCardAttribute *attr)
 	return sf;
 }
 
-static CouchDBStructField *
+static CouchdbStructField *
 contact_postal_address_to_struct_field (EContact *contact, EContactField field, const char *description)
 {
-	CouchDBStructField *sf = NULL;
+	CouchdbStructField *sf = NULL;
 	EContactAddress *contact_address;
 
 	contact_address = e_contact_get (contact, field);
@@ -584,7 +585,7 @@ contact_postal_address_to_struct_field (EContact *contact, EContactField field,
 	return sf;
 }
 
-static CouchDBStructField *
+static CouchdbStructField *
 contact_url_to_struct_field (EVCardAttribute *attr, const gchar *description)
 {
 	const gchar *address, *uuid;
@@ -612,7 +613,7 @@ contact_url_to_struct_field (EVCardAttribute *attr, const gchar *description)
 	return couchdb_document_contact_url_new (uuid, address, description);
 }
 
-static CouchDBStructField *
+static CouchdbStructField *
 contact_im_to_struct_field (EVCardAttribute *attr, const gchar *protocol)
 {
 	const gchar *address, *description = NULL, *uuid = NULL;
@@ -650,16 +651,16 @@ contact_im_to_struct_field (EVCardAttribute *attr, const gchar *protocol)
 	return couchdb_document_contact_im_new (uuid, address, description, protocol);
 }
 
-static CouchDBDocument *
+static CouchdbDocument *
 couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *contact)
 {
 	EContactDate *dt;
 	GSList *list;
 	GList *attr_list, *al;
 	const char *str;
-	CouchDBDocument *document;
+	CouchdbDocument *document;
 	gint i;
-	CouchDBStructField *postal_address;
+	CouchdbStructField *postal_address;
 
 	/* create the CouchDBDocument to put on the database */
 	document = couchdb_document_contact_new (couchdb_backend->couchdb);
@@ -690,7 +691,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	list = NULL;
 	attr_list = e_contact_get_attributes (contact, E_CONTACT_EMAIL);
 	for (al = attr_list; al != NULL; al = al->next) {
-		CouchDBStructField *sf;
+		CouchdbStructField *sf;
 		EVCardAttribute *attr = (EVCardAttribute *) al->data;
 
 		sf = contact_email_to_struct_field (attr);
@@ -709,7 +710,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	list = NULL;
 	attr_list = e_vcard_get_attributes (E_VCARD (contact));
 	for (al = attr_list; al != NULL; al = al->next) {
-		CouchDBStructField *sf;
+		CouchdbStructField *sf;
 		EVCardAttribute *attr = (EVCardAttribute *) al->data;
 
 		if (g_strcmp0 (e_vcard_attribute_get_name (attr), EVC_TEL) == 0) {
@@ -751,7 +752,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	list = NULL;
 	attr_list = e_vcard_get_attributes (E_VCARD (contact));
 	for (al = attr_list; al != NULL; al = al->next) {
-		CouchDBStructField *sf = NULL;
+		CouchdbStructField *sf = NULL;
 		EVCardAttribute *attr = (EVCardAttribute *) al->data;
 
 		if (g_strcmp0 (e_vcard_attribute_get_name (attr),
@@ -776,7 +777,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	list = NULL;
 	attr_list = e_vcard_get_attributes (E_VCARD (contact));
 	for (al = attr_list; al != NULL; al = al->next) {
-		CouchDBStructField *sf = NULL;
+		CouchdbStructField *sf = NULL;
 		EVCardAttribute *attr = (EVCardAttribute *) al->data;
 
 		if (g_strcmp0 (e_vcard_attribute_get_name (attr),
@@ -840,7 +841,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	/* application annotations */
 	str = e_vcard_attribute_get_value (e_vcard_get_attribute (E_VCARD (contact), COUCHDB_APPLICATION_ANNOTATIONS_PROP));
 	if (str) {
-		CouchDBStructField *annotations;
+		CouchdbStructField *annotations;
 
 		annotations = couchdb_struct_field_new_from_string (str);
 		if (annotations) {
@@ -853,7 +854,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 }
 
 static void
-document_updated_cb (CouchDB *couchdb, const char *dbname, CouchDBDocument *document, gpointer user_data)
+document_updated_cb (CouchdbSession *couchdb, const char *dbname, CouchdbDocument *document, gpointer user_data)
 {
 	char *vcard;
 	EContact *contact;
@@ -876,7 +877,7 @@ document_updated_cb (CouchDB *couchdb, const char *dbname, CouchDBDocument *docu
 }
 
 static void
-document_deleted_cb (CouchDB *couchdb, const char *dbname, const char *docid, gpointer user_data)
+document_deleted_cb (CouchdbSession *couchdb, const char *dbname, const char *docid, gpointer user_data)
 {
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (user_data);
 
@@ -893,10 +894,8 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 {
 	gchar *uri;
 	const gchar *property;
-	CouchDBDatabaseInfo *db_info;
+	CouchdbDatabaseInfo *db_info;
 	GError *error = NULL;
-	gboolean enable_oauth = FALSE;
-	char *oauth_c_key = NULL, *oauth_c_secret = NULL, *oauth_t_key = NULL, *oauth_t_secret = NULL;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_COUCHDB (couchdb_backend), GNOME_Evolution_Addressbook_OtherError);
@@ -912,109 +911,31 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 
 	property = e_source_get_property (source, "couchdb_instance");
 	if (g_strcmp0 (property, "user") == 0) {
-		DBusGConnection *bus;
-		DBusGProxy *proxy;
-		gint port;
-		gboolean success;
-
-		/* Get the port via the org.desktopcouch.CouchDB interface */
-		error = NULL;
-		bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-		if (error) {
-			g_warning ("Couldn't get session bus: %s", error->message);
-			g_error_free (error);
-
-			return GNOME_Evolution_Addressbook_NoSuchBook;
-		}
-
-		proxy = dbus_g_proxy_new_for_name (bus,
-						   "org.desktopcouch.CouchDB",
-						   "/",
-						   "org.desktopcouch.CouchDB");
-
-		error = NULL;
-		success = dbus_g_proxy_call (proxy, "getPort", &error, G_TYPE_INVALID,
-					     G_TYPE_INT, &port, G_TYPE_INVALID);
-
-		/* Free memory */
-		g_object_unref (G_OBJECT (proxy));
-		dbus_g_connection_unref (bus);
-
-		if (success) {
-			GnomeKeyringAttributeList *attrs;
-			GnomeKeyringResult result;
-			GList *items_found;
-
-			uri = g_strdup_printf ("http://127.0.0.1:%d", port);
-
-			/* Get OAuth tokens from GnomeKeyring */
-			attrs = gnome_keyring_attribute_list_new ();
-			gnome_keyring_attribute_list_append_string (attrs, "desktopcouch", "oauth");
-
-			result = gnome_keyring_find_items_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
-								attrs, &items_found);
-			if (result == GNOME_KEYRING_RESULT_OK && items_found != NULL) {
-				gchar **items;
-				GnomeKeyringFound *first_item = (GnomeKeyringFound *) items_found->data;
-
-				items = g_strsplit (first_item->secret, ":", 4);
-				if (items) {
-					oauth_c_key = g_strdup (items[0]);
-					oauth_c_secret = g_strdup (items[1]);
-					oauth_t_key = g_strdup (items[2]);
-					oauth_t_secret = g_strdup (items[3]);
-					g_strfreev (items);
-
-					enable_oauth = TRUE;
-				}
-
-				gnome_keyring_found_list_free (items_found);
-			} else {
-				g_warning ("Could not get OAuth tokens from keyring: %s",
-					   gnome_keyring_result_to_message (result));
-			}
-
-			/* Free memory */
-			gnome_keyring_attribute_list_free (attrs);
-		} else {
-			g_warning ("Couldn't get port for desktopcouch: %s", error->message);
-			g_error_free (error);
-
+		if (! (couchdb_backend->couchdb = COUCHDB_SESSION (desktopcouch_session_new ()))) {
+			g_warning ("Could not create DesktopcouchSession object");
 			return GNOME_Evolution_Addressbook_NoSuchBook;
 		}
 
 		couchdb_backend->using_desktopcouch = TRUE;
+	} else {
+		if (g_strcmp0 (property, "remote") == 0)
+			uri = g_strdup_printf ("http://%s", e_source_get_property (source, "couchdb_remote_server"));
+		else
+			uri = g_strdup ("http://127.0.0.1:5984");
 
-	} else if (g_strcmp0 (property, "remote") == 0)
-		uri = g_strdup_printf ("http://%s", e_source_get_property (source, "couchdb_remote_server"));
-	else
-		uri = g_strdup ("http://127.0.0.1:5984");
+		if (! (couchdb_backend->couchdb = couchdb_session_new (uri))) {
+			g_free (uri);
+			return GNOME_Evolution_Addressbook_OtherError;
+		}
 
-	if (! (couchdb_backend->couchdb = couchdb_new (uri))) {
 		g_free (uri);
-		return GNOME_Evolution_Addressbook_OtherError;
 	}
 
-	if (enable_oauth) {
-		couchdb_enable_oauth (couchdb_backend->couchdb,
-				      oauth_c_key,
-				      oauth_c_secret,
-				      oauth_t_key,
-				      oauth_t_secret);
-
-		g_free (oauth_c_key);
-		g_free (oauth_c_secret);
-		g_free (oauth_t_key);
-		g_free (oauth_t_secret);
-	}
-
-	g_free (uri);
-
 	/* check if only_if_exists */
 	error = NULL;
-	db_info = couchdb_get_database_info (couchdb_backend->couchdb,
-					     couchdb_backend->dbname,
-					     &error);
+	db_info = couchdb_session_get_database_info (couchdb_backend->couchdb,
+						     couchdb_backend->dbname,
+						     &error);
 	if (!db_info) {
 		if (error) {
 			g_warning ("Could not get CouchDB database info: %s", error->message);
@@ -1026,9 +947,9 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 		
 		/* if it does not exist, create it */
 		error = NULL;
-		if (!couchdb_create_database (couchdb_backend->couchdb,
-					      couchdb_backend->dbname,
-					      &error)) {
+		if (!couchdb_session_create_database (couchdb_backend->couchdb,
+						      couchdb_backend->dbname,
+						      &error)) {
 			g_warning ("Could not create 'contacts' database: %s", error->message);
 			g_error_free (error);
 
@@ -1047,7 +968,7 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 			  G_CALLBACK (document_updated_cb), couchdb_backend);
 	g_signal_connect (G_OBJECT (couchdb_backend->couchdb), "document_deleted",
 			  G_CALLBACK (document_deleted_cb), couchdb_backend);
-	couchdb_listen_for_changes (couchdb_backend->couchdb, couchdb_backend->dbname);
+	couchdb_session_listen_for_changes (couchdb_backend->couchdb, couchdb_backend->dbname);
 
 	return GNOME_Evolution_Addressbook_Success;
 }
@@ -1069,7 +990,7 @@ e_book_backend_couchdb_get_static_capabilities (EBookBackend *backend)
 }
 
 static EContact *
-put_document (EBookBackendCouchDB *couchdb_backend, CouchDBDocument *document)
+put_document (EBookBackendCouchDB *couchdb_backend, CouchdbDocument *document)
 {
 	GError *error = NULL;
 
@@ -1101,7 +1022,7 @@ e_book_backend_couchdb_create_contact (EBookBackend *backend,
 				       const char *vcard)
 {
 	EContact *contact, *new_contact;
-	CouchDBDocument *document;
+	CouchdbDocument *document;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	contact = e_contact_new_from_vcard (vcard);
@@ -1139,14 +1060,14 @@ e_book_backend_couchdb_remove_contacts (EBookBackend *backend,
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	for (l = id_list; l != NULL; l = l->next) {
-		CouchDBDocument *document;
+		CouchdbDocument *document;
 		GError *error = NULL;
 		const gchar *uid = (const gchar *) l->data;
 
 		document = couchdb_document_get (couchdb_backend->couchdb, couchdb_backend->dbname, uid, &error);
 		if (document) {
 			if (couchdb_backend->using_desktopcouch) {
-				CouchDBStructField *app_annotations, *u1_annotations, *private_annotations;
+				CouchdbStructField *app_annotations, *u1_annotations, *private_annotations;
 
 				/* For desktopcouch, we don't remove contacts, we just
 				 * mark them as deleted */
@@ -1218,7 +1139,7 @@ e_book_backend_couchdb_modify_contact (EBookBackend *backend,
 				       const char *vcard)
 {
 	EContact *contact, *new_contact;
-	CouchDBDocument *document;
+	CouchdbDocument *document;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	contact = e_contact_new_from_vcard (vcard);
@@ -1253,7 +1174,7 @@ e_book_backend_couchdb_get_contact (EBookBackend *backend,
 				    const char *id)
 {
 	GError *error = NULL;
-	CouchDBDocument *document;
+	CouchdbDocument *document;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	document = couchdb_document_get (couchdb_backend->couchdb,
@@ -1285,16 +1206,24 @@ e_book_backend_couchdb_get_contact_list (EBookBackend *backend,
 	GSList *doc_list, *sl;
 	GList *contacts = NULL;
 	GError *error = NULL;
+	EBookBackendSExp *card_sexp = NULL;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
+	card_sexp = e_book_backend_sexp_new (query);
+	if (!card_sexp) {
+		/* XXX this needs to be an invalid query error of some sort*/
+		e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
+		return;
+	}
+
 	/* Get the list of documents from CouchDB */
-	doc_list = couchdb_list_documents (couchdb_backend->couchdb,
-					   couchdb_backend->dbname,
-					   &error);
+	doc_list = couchdb_session_list_documents (couchdb_backend->couchdb,
+						   couchdb_backend->dbname,
+						   &error);
 	for (sl = doc_list; sl != NULL; sl = sl->next) {
 		char *vcard;
-		CouchDBDocument *document;
-		CouchDBDocumentInfo *doc_info = (CouchDBDocumentInfo *) sl->data;
+		CouchdbDocument *document;
+		CouchdbDocumentInfo *doc_info = (CouchdbDocumentInfo *) sl->data;
 
 		/* Retrieve this document */
 		error = NULL;
@@ -1306,13 +1235,18 @@ e_book_backend_couchdb_get_contact_list (EBookBackend *backend,
 			continue;
 
 		vcard = vcard_from_couch_document (document);
-		if (vcard != NULL)
-			contacts = g_list_prepend (contacts, vcard);
+		if (vcard != NULL) {
+			if (e_book_backend_sexp_match_vcard (card_sexp, vcard))
+				contacts = g_list_prepend (contacts, vcard);
+			else
+				g_free (vcard);
+		}
 
 		g_object_unref (G_OBJECT (document));
 	}
 
-	couchdb_free_document_list (doc_list);
+	g_object_unref (card_sexp);
+	couchdb_session_free_document_list (doc_list);
 
 	e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_Success, contacts);
 }
@@ -1330,13 +1264,13 @@ e_book_backend_couchdb_start_book_view (EBookBackend *backend,
 	sexp = e_data_book_view_get_card_sexp  (book_view);
 
 	/* Get the list of documents from CouchDB */
-	doc_list = couchdb_list_documents (couchdb_backend->couchdb,
-					   couchdb_backend->dbname,
-					   &error);
+	doc_list = couchdb_session_list_documents (couchdb_backend->couchdb,
+						   couchdb_backend->dbname,
+						   &error);
 	for (sl = doc_list; sl != NULL; sl = sl->next) {
-		CouchDBDocument *document;
+		CouchdbDocument *document;
 		char *vcard;
-		CouchDBDocumentInfo *doc_info = (CouchDBDocumentInfo *) sl->data;
+		CouchdbDocumentInfo *doc_info = (CouchdbDocumentInfo *) sl->data;
 
 		/* Retrieve this document */
 		error = NULL;
@@ -1359,7 +1293,7 @@ e_book_backend_couchdb_start_book_view (EBookBackend *backend,
 		g_object_unref (G_OBJECT (document));
 	}
 
-	couchdb_free_document_list (doc_list);
+	couchdb_session_free_document_list (doc_list);
 	e_data_book_view_notify_complete (book_view, GNOME_Evolution_Addressbook_Success);
 }
 
